(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.VueDownshift = {})));
}(this, (function (exports) { 'use strict';

var nestRE = /^(attrs|props|on|nativeOn|class|style|hook)$/;

var babelHelperVueJsxMergeProps = function mergeJSXProps (objs) {
  return objs.reduce(function (a, b) {
    var aa, bb, key, nestedKey, temp;
    for (key in b) {
      aa = a[key];
      bb = b[key];
      if (aa && nestRE.test(key)) {
        // normalize class
        if (key === 'class') {
          if (typeof aa === 'string') {
            temp = aa;
            a[key] = aa = {};
            aa[temp] = true;
          }
          if (typeof bb === 'string') {
            temp = bb;
            b[key] = bb = {};
            bb[temp] = true;
          }
        }
        if (key === 'on' || key === 'nativeOn' || key === 'hook') {
          // merge functions
          for (nestedKey in bb) {
            aa[nestedKey] = mergeFn(aa[nestedKey], bb[nestedKey]);
          }
        } else if (Array.isArray(aa)) {
          a[key] = aa.concat(bb);
        } else if (Array.isArray(bb)) {
          a[key] = [aa].concat(bb);
        } else {
          for (nestedKey in bb) {
            aa[nestedKey] = bb[nestedKey];
          }
        }
      } else {
        a[key] = b[key];
      }
    }
    return a
  }, {})
};

function mergeFn (a, b) {
  return function () {
    a.apply(this, arguments);
    b.apply(this, arguments);
  }
}

function findParent(finder, node, rootNode) {
  if (node !== null && node !== rootNode.parentNode) {
    if (finder(node)) {
      if (node === document.body && node.scrollTop === 0) {
        // in chrome body.scrollTop always return 0
        return document.documentElement;
      }
      return node;
    } else {
      return findParent(finder, node.parentNode, rootNode);
    }
  } else {
    return null;
  }
}

var getClosestScrollParent = findParent.bind(null, function (node) {
  return node.scrollHeight > node.clientHeight;
});

// eslint-disable-next-line complexity
function scrollIntoView(node, rootNode) {
  var scrollParent = getClosestScrollParent(node, rootNode);
  if (scrollParent === null) {
    return;
  }
  var scrollParentStyles = getComputedStyle(scrollParent);
  var scrollParentRect = scrollParent.getBoundingClientRect();
  var scrollParentBorderTopWidth = parseInt(scrollParentStyles.borderTopWidth, 10);
  var scrollParentBorderBottomWidth = parseInt(scrollParentStyles.borderBottomWidth, 10);
  var bordersWidth = scrollParentBorderTopWidth + scrollParentBorderBottomWidth;
  var scrollParentTop = scrollParentRect.top + scrollParentBorderTopWidth;
  var nodeRect = node.getBoundingClientRect();

  if (nodeRect.top < 0) {
    // the item above view
    scrollParent.scrollTop += nodeRect.top;
    return;
  }

  if (nodeRect.top > 0 && scrollParentRect.top < 0) {
    if (scrollParentRect.bottom > 0 && nodeRect.bottom + bordersWidth > scrollParentRect.bottom) {
      // the item is below scrollable area
      scrollParent.scrollTop += nodeRect.bottom - scrollParentRect.bottom + bordersWidth;
    }
    // item and parent top are on different sides of view top border (do nothing)
    return;
  }

  var nodeOffsetTop = nodeRect.top + scrollParent.scrollTop;
  var nodeTop = nodeOffsetTop - scrollParentTop;
  if (nodeTop < scrollParent.scrollTop) {
    // the item is above the scrollable area
    scrollParent.scrollTop = nodeTop;
  } else if (nodeTop + nodeRect.height + bordersWidth > scrollParent.scrollTop + scrollParentRect.height) {
    // the item is below the scrollable area
    scrollParent.scrollTop = nodeTop + nodeRect.height - scrollParentRect.height + bordersWidth;
  }
  // the item is within the scrollable area (do nothing)
}

var sortBy = function (obj, fn) {
  return obj.map(function (value, index) {
    return {
      value: value,
      index: index,
      criteria: fn(value)
    };
  }).sort(
  // eslint-disable-next-line complexity
  function (_ref, _ref2) {
    var leftCriteria = _ref.criteria,
        leftIndex = _ref.index;
    var rightCriteria = _ref2.criteria,
        rightIndex = _ref2.index;

    if (leftCriteria !== rightCriteria) {
      if (leftCriteria > rightCriteria) return 1;
      if (leftCriteria < rightCriteria) return -1;
    }

    return leftIndex - rightIndex;
  }).map(function (_ref3) {
    var value = _ref3.value;
    return value;
  });
};

function isString(value) {
  return typeof value === 'string';
}

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var Downshift$1 = {
  name: 'Downshift',
  props: {
    items: {
      required: true,
      type: Array
    },

    rootProps: {
      type: Object
    },

    /**
     * Controllable props
     */
    isOpen: {},
    inputValue: { type: String },
    selectedItem: {},
    highlightedIndex: Number,

    itemToString: {
      type: Function,
      default: function _default(i) {
        return i == null ? '' : String(i);
      }
    },

    placeholderizeValue: {
      default: false
    }
  },

  data: function data() {
    return {
      isMouseDown: false,
      id: btoa(Math.random()).substring(0, 12),

      // eslint-disable-next-line camelcase
      internal_isOpen: false,
      // eslint-disable-next-line camelcase
      internal_inputValue: '',
      // eslint-disable-next-line camelcase
      internal_selectedItem: null,
      // eslint-disable-next-line camelcase
      internal_highlightedIndex: null
    };
  },

  watch: {
    selectedItem: function selectedItem(newSelectedItem) {
      this.setState({
        inputValue: this.itemToString(newSelectedItem)
      });
    },
    items: function items() {
      this.setState({
        inputValue: this.itemToString(this.mergedState.selectedItem)
      });
    }
  },

  computed: {
    mergedState: function mergedState() {
      var _this = this;

      return Object.keys(this.$options.data()).filter(function (key) {
        return key.indexOf('internal_') === 0;
      }).map(function (key) {
        return key.split('_')[1];
      }).reduce(function (state, key) {
        return _extends({}, state, defineProperty({}, key, _this.isControlledProp(key) ? _this.$props[key] : _this['internal_' + key]));
      }, {});
    },
    internalItemCount: function internalItemCount() {
      return this.items.length;
    }
  },

  created: function created() {
    this.setState({
      inputValue: this.itemToString(this.mergedState.selectedItem)
    });
  },
  mounted: function mounted() {
    window.addEventListener('mousedown', this.handleWindowMousedown);
    window.addEventListener('mouseup', this.handleWindowMouseup);
  },
  beforeDestroy: function beforeDestroy() {
    window.removeEventListener('mousedown', this.handleWindowMousedown);
    window.removeEventListener('mouseup', this.handleWindowMouseup);
  },


  methods: {
    handleWindowMousedown: function handleWindowMousedown() {
      this.isMouseDown = true;
    },
    handleWindowMouseup: function handleWindowMouseup(event) {
      this.isMouseDown = false;

      if ((event.target === this.$refs.rootNode || !this.$refs.rootNode.contains(event.target)) && this.mergedState.isOpen) {
        if (!this.isMouseDown) {
          this.reset();
        }
      }
    },
    keyDownArrowDown: function keyDownArrowDown(event) {
      event.preventDefault();
      var amount = event.shiftKey ? 5 : 1;

      if (this.mergedState.isOpen) {
        this.changeHighlightedIndex(amount);
      } else {
        this.setState({
          isOpen: true
        });

        this.setHighlightedIndex();
      }
    },
    keyDownArrowUp: function keyDownArrowUp(event) {
      event.preventDefault();
      var amount = event.shiftKey ? -5 : -1;

      if (this.mergedState.isOpen) {
        this.changeHighlightedIndex(amount);
      } else {
        this.setState({
          isOpen: true
        });

        this.setHighlightedIndex();
      }
    },
    keyDownEnter: function keyDownEnter(event) {
      if (this.mergedState.isOpen) {
        event.preventDefault();
        this.selectHighlightedItem();
      }
    },
    keyDownEscape: function keyDownEscape(event) {
      event.preventDefault();
      this.reset();
    },
    selectHighlightedItem: function selectHighlightedItem() {
      return this.selectItemAtIndex(this.mergedState.highlightedIndex);
    },
    selectItemAtIndex: function selectItemAtIndex(itemIndex) {
      var item = this.items[itemIndex];

      if (item == null) {
        return;
      }

      this.selectItem(item);
    },
    selectItem: function selectItem(item) {
      this.setState({
        isOpen: false,
        highlightedIndex: null,
        selectedItem: item,
        inputValue: this.itemToString(item)
      });
    },


    // eslint-disable-next-line complexity
    changeHighlightedIndex: function changeHighlightedIndex(moveAmount) {
      this.internalItemCount - 1;


      if (this.internalItemCount < 0) {
        return;
      }

      var highlightedIndex = this.mergedState.highlightedIndex;


      var baseIndex = highlightedIndex;

      if (baseIndex === null) {
        baseIndex = moveAmount > 0 ? -1 : this.internalItemCount + 1;
      }

      var newIndex = baseIndex + moveAmount;

      if (newIndex < 0) {
        newIndex = this.internalItemCount;
      } else if (newIndex > this.internalItemCount) {
        newIndex = 0;
      }

      this.setHighlightedIndex(newIndex);
    },
    setHighlightedIndex: function setHighlightedIndex() {
      var highlightedIndex = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

      this.setState({
        highlightedIndex: highlightedIndex
      });

      scrollIntoView(document.getElementById(this.getItemId(highlightedIndex)), this.$refs.rootNode);
    },
    reset: function reset() {
      arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var selectedItem = this.mergedState.selectedItem;


      this.setState({
        isOpen: false,
        highlightedIndex: null,
        inputValue: this.itemToString(selectedItem)
      });
    },
    getItemId: function getItemId(index) {
      return this.id + '-item-' + index;
    },
    getItemProps: function getItemProps(_ref) {
      var index = _ref.index,
          item = _ref.item;

      if (index === undefined) {
        index = this.items.indexOf(item);
      }

      return {
        id: this.getItemId(index)
      };
    },
    getItemEvents: function getItemEvents(_ref2) {
      var index = _ref2.index,
          item = _ref2.item;

      if (index === undefined) {
        index = this.items.indexOf(item);
      }

      var vm = this;

      return {
        mouseenter: function mouseenter() {
          vm.setHighlightedIndex(index);
        },
        click: function click(event) {
          event.stopPropagation();
          vm.selectItemAtIndex(index);
        }
      };
    },
    getInputProps: function getInputProps() {
      var _mergedState = this.mergedState,
          inputValue = _mergedState.inputValue,
          selectedItem = _mergedState.selectedItem;


      return defineProperty({}, selectedItem === this.placeholderizeValue || !selectedItem ? 'placeholder' : 'value', inputValue || '');
    },
    getInputEvents: function getInputEvents() {
      var _this2 = this;

      return {
        input: function input(event) {
          _this2.setState({
            isOpen: true,
            inputValue: event.target.value
          });
        },

        keydown: function keydown(event) {
          if (event.key && _this2['keyDown' + event.key]) {
            // eslint-disable-next-line no-useless-call
            _this2['keyDown' + event.key].call(_this2, event);
          }
        },

        blur: function blur() {
          if (!_this2.isMouseDown) {
            _this2.reset();
          }
        }
      };
    },
    getHelpersAndState: function getHelpersAndState() {
      var getItemProps = this.getItemProps,
          getItemEvents = this.getItemEvents,
          getInputProps = this.getInputProps,
          getInputEvents = this.getInputEvents,
          itemToString = this.itemToString,
          openMenu = this.openMenu,
          setState = this.setState;


      return _extends({
        getItemProps: getItemProps,
        getItemEvents: getItemEvents,

        getInputProps: getInputProps,
        getInputEvents: getInputEvents,

        itemToString: itemToString,
        setState: setState,
        openMenu: openMenu
      }, this.mergedState);
    },
    openMenu: function openMenu() {
      this.setState({ isOpen: true });
    },
    isControlledProp: function isControlledProp(prop) {
      return this.$props[prop] !== undefined;
    },
    setState: function setState(stateToSet) {
      var _this3 = this;

      Object.keys(stateToSet).map(function (key) {
        return _this3.isControlledProp(key) ? _this3.$emit(key + 'Change', stateToSet[key]) : _this3['internal_' + key] = stateToSet[key];
      });

      this.$emit('stateChange', this.mergedState);
    }
  },

  render: function render(h) {
    return h(
      'div',
      babelHelperVueJsxMergeProps([this.rootProps, { ref: 'rootNode' }]),
      [this.$scopedSlots.default && this.$scopedSlots.default(_extends({}, this.getHelpersAndState(), {
        getHelpersAndState: this.getHelpersAndState
      }))]
    );
  }
};

var SingleDownshiftItem = {
  props: ['item', 'index', 'getHelpersAndState', 'items', 'getItemProps'],

  render: function render(h) {
    var _ref;

    var _getHelpersAndState = this.getHelpersAndState(),
        getItemProps = _getHelpersAndState.getItemProps,
        getItemEvents = _getHelpersAndState.getItemEvents,
        highlightedIndex = _getHelpersAndState.highlightedIndex,
        selectedItem = _getHelpersAndState.selectedItem,
        itemToString = _getHelpersAndState.itemToString;

    return h(
      'div',
      babelHelperVueJsxMergeProps([{
        attrs: _extends({}, getItemProps({
          item: this.item
        }), this.getItemProps({ item: this.item }))
      }, {
        on: getItemEvents({
          item: this.item
        })
      }, {
        'class': (_ref = {}, defineProperty(_ref, 'ct-select-dropdown-item', true), defineProperty(_ref, 'active', highlightedIndex === this.index), defineProperty(_ref, 'selected', selectedItem === this.item), _ref)
      }]),
      [itemToString(this.item) || this.items[this.item]]
    );
  }
};

var selectFromChoicesWithGroups = {
  name: 'CtSelectFromChoices',

  props: {
    value: {},
    choices: {},

    /**
     * only first level options right now
     */
    choicesOrder: {
      type: Array
    },

    getItemProps: {
      type: Function,
      default: function _default() {
        return {};
      }
    },

    getInputProps: {
      type: Function,
      default: function _default() {
        return {};
      }
    },

    placeholderizeValue: {
      default: false
    },

    getInputEvents: {
      type: Function,
      default: function _default() {
        return {};
      }
    }
  },

  computed: {
    itemsWithoutGroups: function () {
      var _this = this;

      var itemsWithoutGroups = {};

      var computeItems = function (items) {
        sortBy(Object.keys(items), function (item) {
          return item;
        }).forEach(function (item) {
          if (items[item].choices) {
            computeItems(items[item].choices);
          } else {
            itemsWithoutGroups[item] = _this.itemToString(item, items);
          }
        });
      };

      computeItems(this.choices);

      return itemsWithoutGroups;
    }
  },

  methods: {
    filterItemsBy: function filterItemsBy(items, getHelpersAndState) {
      var _this2 = this;

      var result = {};

      Object.keys(items).forEach(function (i) {
        if (!isString(items[i]) && items[i].choices) {
          // We have a group
          var newChoices = _this2.filterItemsBy(items[i].choices, getHelpersAndState);

          if (Object.keys(newChoices).length > 0) {
            result[i] = items[i];
          }
        } else {
          if (_this2.hasSingleItem(i, getHelpersAndState)) {
            result[i] = items[i];
          }
        }
      });

      return result;
    },
    itemToString: function itemToString(item) {
      var _this3 = this;

      var choices = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.choices;

      if (choices[item]) {
        // if (!item) return

        return (choices[item].text ? choices[item].text : choices[item]) || '';
      } else {
        return Object.keys(choices).reduce(function (currentLabel, currentMaybeGroup) {
          if (currentLabel) return currentLabel;

          if (isString(choices[currentMaybeGroup]) || !choices[currentMaybeGroup].choices) {
            return false;
          }

          // We have a group
          return _this3.itemToString(item, choices[currentMaybeGroup].choices);
        }, false);
      }
    },
    hasSingleItem: function hasSingleItem(item, _ref2) {
      var inputValue = _ref2.inputValue,
          selectedItem = _ref2.selectedItem,
          itemToString = _ref2.itemToString;

      return itemToString(selectedItem) === inputValue || !inputValue || itemToString(item).toLowerCase().includes(inputValue.toLowerCase());
    }
  },

  render: function render() {
    var _this4 = this;

    var h = arguments[0];

    return h(
      Downshift$1,
      {
        attrs: {
          placeholderizeValue: this.placeholderizeValue,
          items: Object.keys(this.itemsWithoutGroups),
          itemToString: this.itemToString,
          selectedItem: this.value,
          rootProps: {
            class: 'ct-option-select'
          }
        },
        on: {
          'selectedItemChange': function selectedItemChange(newSelectedItem) {
            return _this4.$emit('input', newSelectedItem);
          }
        },

        scopedSlots: {
          default: function _default(_ref3) {
            var getInputEvents = _ref3.getInputEvents,
                getInputProps = _ref3.getInputProps,
                getItemProps = _ref3.getItemProps,
                getItemEvents = _ref3.getItemEvents,
                itemToString = _ref3.itemToString,
                openMenu = _ref3.openMenu,
                getHelpersAndState = _ref3.getHelpersAndState,
                isOpen = _ref3.isOpen,
                inputValue = _ref3.inputValue,
                highlightedIndex = _ref3.highlightedIndex,
                selectedItem = _ref3.selectedItem;
            return [h(
              'div',
              { 'class': 'ct-select-input' },
              [h(
                'input',
                babelHelperVueJsxMergeProps([{
                  on: _extends({}, getInputEvents(), {
                    focus: openMenu
                  }, _this4.getInputEvents(_this4.value, getHelpersAndState()))
                }, {
                  domProps: _extends({}, getInputProps(), _this4.getInputProps(_this4.value, getHelpersAndState()))
                }, {
                  attrs: {
                    placeholder: 'Select value...'
                  }
                }]),
                []
              )]
            )].concat(isOpen && h(
              'div',
              { 'class': 'ct-select-dropdown' },
              [Object.keys(_this4.filterItemsBy(_this4.choices, getHelpersAndState())).length === 0 && h(
                'div',
                { 'class': 'ct-select-no-results' },
                ['No results']
              ), (_this4.choicesOrder ? function (t, c) {
                return c(t);
              }(_this4.filterItemsBy(_this4.choices, getHelpersAndState()), function (ourChoices) {
                return _this4.choicesOrder.filter(function (key) {
                  return !!ourChoices[key];
                });
              }) : sortBy(Object.keys(_this4.filterItemsBy(_this4.choices, getHelpersAndState())), function (item) {
                return item;
              })).map(function (item) {
                return _this4.choices[item].choices ? h(
                  'div',
                  { 'class': 'ct-select-dropdown-group' },
                  [h(
                    'h2',
                    null,
                    [_this4.choices[item].attr ? _this4.choices[item].attr.label : item]
                  ), Object.keys(_this4.choices[item].choices).map(function (nestedItem) {
                    return h(
                      SingleDownshiftItem,
                      {
                        attrs: {
                          item: nestedItem,
                          index: Object.keys(_this4.itemsWithoutGroups).indexOf(nestedItem),

                          items: _this4.choices[item].choices,
                          getHelpersAndState: getHelpersAndState,
                          getItemProps: _this4.getItemProps
                        },
                        key: nestedItem },
                      []
                    );
                  })]
                ) : h(
                  SingleDownshiftItem,
                  {
                    attrs: {
                      item: item,
                      index: Object.keys(_this4.itemsWithoutGroups).indexOf(item),

                      items: _this4.choices,
                      getHelpersAndState: getHelpersAndState,
                      getItemProps: _this4.getItemProps
                    },
                    key: item },
                  []
                );
              })]
            )).concat(_this4.$scopedSlots.default && _this4.$scopedSlots.default());
          }
        }
      },
      []
    );
  }
};

exports.SelectFromChoices = selectFromChoicesWithGroups;
exports['default'] = Downshift$1;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=vue-downshift.umd.js.map
